
/* Chrysalide - Outil d'analyse de fichiers binaires
 * targetable.c - prototypes pour l'équivalent Python du fichier "arch/operands/targetable.c"
 *
 * Copyright (C) 2018 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "targetable.h"


#include <pygobject.h>


#include <i18n.h>


#include <arch/operands/targetable.h>


#include "../processor.h"
#include "../vmpa.h"
#include "../../access.h"
#include "../../helpers.h"
#include "../../format/format.h"



#define TARGETABLE_OPERAND_DOC                                              \
    "The TargetableOperand interface depicts operands which can drive to"   \
    " another location.\n"                                                  \
    "\n"                                                                    \
    "By instance, an immediate value can target a given address into"       \
    " some function code."



/* Obtient l'adresse de la cible visée par un opérande. */
static PyObject *py_targetable_operand_get_addr(PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = contenu binaire à manipuler.                          *
*                args = non utilisé ici.                                      *
*                                                                             *
*  Description : Obtient l'adresse de la cible visée par un opérande.         *
*                                                                             *
*  Retour      : Localisation de la cible ou None.                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_targetable_operand_get_addr(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Instance à retourner        */
    vmpa2t *src;                            /* Version native d'adresse    */
    GBinFormat *format;                     /* Instance GLib du format     */
    GArchProcessor *proc;                   /* Instance GLib de l'archi.   */
    int ret;                                /* Bilan de lecture des args.  */
    GTargetableOperand *operand;            /* Instance à manipuler        */
    vmpa2t addr;                            /* Localisation à cibler       */
    bool defined;                           /* Cible définie ?             */

#define TARGETABLE_OPERAND_GET_ADDR_METHOD PYTHON_METHOD_DEF            \
(                                                                       \
    get_addr, "$self, src, format, proc",                               \
    METH_VARARGS, py_targetable_operand,                                \
    "Compute a target address from an operand."                         \
    "\n"                                                                \
    "The following arguments are required:\n"                           \
    "* src is the location of the instruction owning the operand, as a" \
    " pychrysalide.arch.vmpa instance.\n"                               \
    "* format is a pychrysalide.format.BinFormat instance, providing"   \
    " all needed information.\n"                                        \
    "* proc is a pychrysalide.arch.ArchProcessor instance, providing"   \
    " all needed information too.\n"                                    \
    "\n"                                                                \
    "The result is a pychrysalide.arch.vmpa address or None."           \
)

    ret = PyArg_ParseTuple(args, "O&O&O&",
                           convert_any_to_vmpa, &src,
                           convert_to_binary_format, &format,
                           convert_to_arch_processor, &proc);
    if (!ret) return NULL;

    operand = G_TARGETABLE_OPERAND(pygobject_get(self));

    defined = g_targetable_operand_get_addr(operand, src, format, proc, &addr);

    if (defined)
        result = build_from_internal_vmpa(&addr);
    else
    {
        result = Py_None;
        Py_INCREF(result);
    }

    clean_vmpa_arg(src);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_targetable_operand_type(void)
{
    static PyMethodDef py_targetable_operand_methods[] = {
        TARGETABLE_OPERAND_GET_ADDR_METHOD,
        { NULL }
    };

    static PyGetSetDef py_targetable_operand_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_targetable_operand_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.arch.operands.TargetableOperand",
        .tp_basicsize   = sizeof(PyObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = TARGETABLE_OPERAND_DOC,

        .tp_methods     = py_targetable_operand_methods,
        .tp_getset      = py_targetable_operand_getseters

    };

    return &py_targetable_operand_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.....TargetableOperand'.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_targetable_operand_is_registered(void)
{
    PyTypeObject *type;                     /* Type 'TargetableOperand'    */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_targetable_operand_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.arch.operands");

        dict = PyModule_GetDict(module);

        if (!register_interface_for_pygobject(dict, G_TYPE_TARGETABLE_OPERAND, type))
            return false;

    }

    return true;

}
